<?php

namespace Rock_Convert\Inc\Frontend;

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @link       https://rockcontent.com
 * @since      1.0.0
 *
 * @author    Rock Content
 */

class Frontend {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * The text domain of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_text_domain    The text domain of this plugin.
   */
  private $plugin_text_domain;

  /**
   * The name of the UTM campaign
   *
   * @since   1.0.0
   * @access  private
   * @var     string
   */
  private $campaign_name;

  /**
   * Initialize the class and set its properties.
   *
   * @since		1.0.0
   * @param		string $plugin_name       The name of this plugin.
   * @param		string $version    The version of this plugin.
   * @param		string $plugin_text_domain	The text domain of this plugin
   */
  public function __construct( $plugin_name, $version, $plugin_text_domain ) {
    $this->plugin_name = $plugin_name;
    $this->version = $version;
    $this->plugin_text_domain = $plugin_text_domain;
    $this->add_filters();
    $this->campaign_name = "RockContent";
  }

  /**
   * Register the stylesheets for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {

    /**
     * This function is provided for demonstration purposes only.
     *
     * An instance of this class should be passed to the run() function
     * defined in Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/rock-convert-frontend.css', array(), $this->version, 'all' );

  }

  /**
   * Register the JavaScript for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {

    /**
     * This function is provided for demonstration purposes only.
     *
     * An instance of this class should be passed to the run() function
     * defined in Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/rock-convert-frontend.js', array( 'jquery' ), $this->version, false );

  }

  /**
   * Register wordpress filters
   */
  public function add_filters() {
    add_filter( 'the_content', array( $this, 'register_cta_frontend') );
  }

  /**
   * Get CTAs for posts
   *
   * @param $content the post content
   */
  public function register_cta_frontend($content) {
    $custom_content = '';
    $post_id        = get_the_ID();
    $is_single      = is_single();

    if ($is_single) {
      $custom_content .= $this->get_cta('top', $post_id);
    }

    $custom_content .= $content;

    if ($is_single) {
      $custom_content .= $this->get_cta('bottom', $post_id);
    }

    return $custom_content;
  }

  /**
   * Get CTA for post in position
   *
   * @param $position the position of the ad
   * @param $post_id  the id of the post
   */
  public function get_cta($position = "bottom", $post_id) {
    $categories = $this->get_post_categories_ids($post_id);
    $html = '';

    if( !empty($categories) ) {
      $args = array(
        'post_type'   => 'cta',
        'cat'         => implode(",", $categories),
        'post_status' => 'publish',
        'meta_key'    => '_rock_convert_position',
        'meta_value'  => $position,
      );
      $cta_query = new \WP_Query($args);

      if( $cta_query->have_posts() ) {

        while ( $cta_query->have_posts() ) {
          $cta_query->the_post();
          $metas = $this->get_cta_values(get_the_ID(), get_the_title());

          if( !empty( $metas["image_tag"] ) ){
            $html .= $this->build_banner($metas);
          }
        }

        wp_reset_postdata();
      }
    }

    return $html;
  }

  public function get_post_categories_ids($post_id) {
    $cats = array();
    $post_categories = wp_get_post_categories( $post_id );

    foreach($post_categories as $c){
      $cat = get_category( $c );
      $cats[] = $cat->term_id;
    }

    return $cats;
  }

  /**
   * Builds an http query string.
   * @param array $query  // of key value pairs to be used in the query
   * @return string       // http query string.
   **/
  function build_http_query( $query ){

    $query_array = array();

    foreach( $query as $key => $key_value ){

      $query_array[] = urlencode( $key ) . '=' . urlencode( $key_value );

    }

    return implode( '&', $query_array );
  }

  public function build_link($metas) {
    $link     = $metas["url"];
    $connector = (strpos($link, '?') !== false) ? null : "?";
    $params   = array();

    if (!empty($metas["source"])) {
      $params["utm_source"] = $metas["source"];
    }

    if (!empty($metas["medium"])) {
      $params["utm_medium"] = $metas["medium"];
    }

    $params["utm_campaign"] = $this->campaign_name;

    return $link . $connector . $this->build_http_query($params);
  }

  public function build_banner($metas) {
    $html = '';

    $html .= '<div class="rock-convert-banner">';
    $html .= '<a href="' . $this->build_link($metas) . '" target="_blank">';
    $html .= $metas["image_tag"];
    $html .= '</a>';
    $html .= '</div>';

    return $html;
  }

  public function get_cta_values($cta_id, $cta_title) {
    $url      = get_post_meta($cta_id, '_rock_convert_title', true);
    $source   = get_post_meta($cta_id, '_rock_convert_utm_source', true);
    $medium   = get_post_meta($cta_id, '_rock_convert_utm_medium', true);
    $image_id = get_post_meta($cta_id, '_rock_convert_image_media', true);

    $image_tag = null;

    if( !empty($image_id) ) {
      $image_tag = wp_get_attachment_image($image_id, "full", false, array("title" => $cta_title));
    }

    return array(
      'image_tag' => $image_tag,
      'url' => $url,
      'source' => $source,
      'medium' => $medium
    );
  }

}
